import cn.estsh.i3plus.platform.common.convert.ConvertBean
import cn.estsh.i3plus.platform.plugin.datasource.DynamicDataSourceProxy
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil
import cn.estsh.i3plus.pojo.mes.bean.MesComunicationLog
import cn.estsh.i3plus.pojo.mes.bean.MesCcscTask
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrder
import cn.estsh.i3plus.pojo.mes.repository.MesCcscTaskRepository
import cn.estsh.i3plus.pojo.mes.repository.MesComunicationLogRepository
import cn.estsh.i3plus.pojo.mes.repository.MesProduceSnRepository
import cn.estsh.i3plus.pojo.mes.repository.MesWorkOrderRepository
import cn.estsh.i3plus.pojo.wms.dbinterface.WmsInterfaceDataMapper
import lombok.Getter
import lombok.Setter
import org.slf4j.Logger
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate
import org.springframework.util.CollectionUtils

import javax.annotation.Resource
import java.sql.Connection

/**
 * @Description : QMS同步MES_CCSC任务检测结果
 * @Reference :
 * @Author : gsz
 * @CreateDate 2024/5/23 9:58
 * @Modify:
 *   */
class sapToMesCcscTaskResult {

    public static final Logger LOGGER = LoggerFactory.getLogger(sapToMesCcscTaskResult.class)

    @Autowired
    private MesCcscTaskRepository mesCcscTaskRDao;
    @Autowired
    private MesComunicationLogRepository mesComunicationLogRDao;
    @Autowired
    private MesProduceSnRepository mesProduceSnRDao;
    @Autowired
    private MesWorkOrderRepository mesWorkOrderRDao;
    @Resource(name = "yfasDataSource")
    private DynamicDataSourceProxy yfasDataSource;

    @Resource(name = "mesDataSource")
    private DynamicDataSourceProxy mesDataSource;

    @Getter
    @Setter
    private Connection mesConn;

    def filterData(WmsInterfaceDataMapper mapper, List<Map<String, Object>> srcData) {
        if (srcData == null || srcData.size() == 0) {
            LOGGER.error("----QMS同步MES_CCSC任务检测结果 no data to filter----");
            return srcData;
        }
        String organizeCode = mapper.getOrganizeCode();

        for (Map<String, Object> map : srcData) {
            String taskNo = map.get("task_no").toString();
            String taskResult = map.get("task_result").toString(); //1:合格 2:不合格
            //查询 mesCcscTask
            MesCcscTask mesCcscTask = mesCcscTaskRDao.getByProperty(
                    ["taskNo", "organizeCode", "isValid", "isDeleted"] as String[],
                    [taskNo, organizeCode, CommonEnumUtil.IS_VAILD.VAILD.getValue(), CommonEnumUtil.IS_DEAL.NO.getValue()] as Object[]);

            if (null == mesCcscTask) {
                continue;
            }
            LOGGER.info("QMS同步MES_CCSC任务检测结果 taskNo：{}", taskNo);

            //根据任务工单号和物料查工单表
            MesWorkOrder mesWorkOrder = mesWorkOrderRDao.getByProperty(
                    ["workOrderNo", "partNo", "organizeCode", "isValid", "isDeleted"] as String[],
                    [mesCcscTask.getWorkOrderNo(), mesCcscTask.getPartNo(), organizeCode,
                     CommonEnumUtil.IS_VAILD.VAILD.getValue(), CommonEnumUtil.IS_DEAL.NO.getValue()] as Object[]);

            if (null == mesWorkOrder) {
                continue;
            }

            MesComunicationLog comunicationLog = mesComunicationLogRDao.getByProperty(
                    ["workCenterCode", "workOrderNo", "value", "organizeCode", "isValid", "isDeleted"] as String[],
                    [mesWorkOrder.getWorkCenterCode(), mesWorkOrder.getWorkCellCode(), mesCcscTask.getPartNo(),
                     organizeCode, CommonEnumUtil.IS_VAILD.VAILD.getValue(), CommonEnumUtil.IS_DEAL.NO.getValue()] as Object[]);
            //软删记录 不合格时新增一条
            if (null != comunicationLog) {
                mesComunicationLogRDao.deleteWeaklyById(comunicationLog.getId())
            }
            if (taskResult.equals("2")) { //1:合格 2:不合格
                MesComunicationLog mesComunicationLog = new MesComunicationLog();
                mesComunicationLog.setOrganizeCode(organizeCode);
                mesComunicationLog.setWorkCellCode(mesCcscTask.getWorkOrderNo());
                mesComunicationLog.setWorkCenterCode(mesCcscTask.getWorkOrderNo());
                mesComunicationLog.setStatus(1);
                mesComunicationLog.setParameterName(mesCcscTask.getPartNo());
                mesComunicationLog.setParameterDesc(mesCcscTask.getPartNameRdd());
                mesComunicationLog.setValue(mesCcscTask.getPartNo());
                ConvertBean.serviceModelInitialize(mesComunicationLog, "CCSC_SAP");
                mesComunicationLogRDao.save(mesComunicationLog);
                //更新所有条码为可疑品
                List<MesProduceSn> mesProduceSnList = mesProduceSnRDao.findByProperty(
                        ["workCenterCode", "workOrderNo", "partNo", "organizeCode", "isValid", "isDeleted"] as String[],
                        [mesWorkOrder.getWorkCenterCode(), mesWorkOrder.getWorkCellCode(), mesCcscTask.getPartNo(),
                         organizeCode, CommonEnumUtil.IS_VAILD.VAILD.getValue(), CommonEnumUtil.IS_DEAL.NO.getValue()] as Object[]);
                if (!CollectionUtils.isEmpty(mesProduceSnList)) {//todo 可疑品状态?
                    mesProduceSnList.forEach(
                            {
                                sn ->
                                    sn.setQcStatus(30)
                                    ConvertBean.serviceModelUpdate(sn, "CCSC_SAP")
                            }
                    )
                }
            }

        }

        // 过滤逻辑处理完后一定要把数据集返回，否则后续的处理逻辑不会执行
        LOGGER.info("[----QMS同步MES_CCSC任务检测结果srcData:{}", srcData);
        return srcData;
    }

    private List<Map<String, Object>> queryDataTable(String sql, Map<String, Object> params) {
        NamedParameterJdbcTemplate namedJdbcTemplate = new NamedParameterJdbcTemplate(yfasDataSource.getDataSource())

        List<Map<String, Object>> dataMap = namedJdbcTemplate.queryForList(sql, params)

        return dataMap;
    }
}